#!/usr/bin/env python
#    VO LDAP management 
#    Copyright (C) 2001, 2002 California Institute of Technology
#    Author: Conrad D. Steenberg <conrad@hep.caltech.edu>
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#    Requires pygtk and python-ldap and optionally M2Crypto


import sys
import os
import pickle
import string
import time
import traceback

try:
  import gtk
  import _gtk
  import GTK
  import ldap
  import wizard
except ImportError,v:
  print v
  print "\nModule load error:"
  print "The above module cannot be found. Please check that it is installed"
  print "in one of the following directories:"
  for path in sys.path:
    print path
  print
  sys.exit(1)

try:
  import M2Crypto
  from M2Crypto import X509,Err
except: # M2Crypto not found, disable .pem import/export
  M2Crypto=None
  pass  # We just test whether the module exists later on, no need for 
        # more info to be stored

def gtk_process_events():
  while gtk.events_pending():
      gtk.mainiteration(block=gtk.FALSE)

def guess_sn(sn):
    snlist=string.split(sn,' ')
    if len(snlist)<2: return sn,sn
    for i in range(len(snlist)-1,-1,-1):
      try:
        real_sn=snlist[i]
        j=int(real_sn)
      except:
        if real_sn[0] not in string.punctuation:
#          print "1:",sntuple[0],sn,repr(real_sn)
          return sn,real_sn
        pass
#    print "2:", sntuple[0],sn,snlist[0]
    return sn,snlist[-1]


class justlabel(gtk.GtkLabel):
        def __init__(self, label="", xalign=0.,yalign=0.5,
                justify=GTK.JUSTIFY_LEFT,_obj=None):
                if _obj: self._o = _obj; return 
                self._o = _gtk.gtk_label_new(label)
                self.set_alignment(xalign,yalign)
                self.set_justify(justify)

class Settings:
  def_sfile=os.environ['HOME']+'/.VOgrouprc'
  sfile=None
  sfile_o=None
  sfile_rw_ok=0
  written=0
  options={'host':['citcms.cacr.caltech.edu',
                    'certificate.nikhef.nl',
                    'security.fi.infn.it',
                    'tady.ten.cz'],
           'port':['389',
                    '389',
                    '389',
                    '389'],
           'basedn':['dc=es,dc=net,c=us',
                    'o=dutchgrid',
                    'o=infn,c=it',
                    'o=cesnet'],
           'bindname':['cn=manager',
                       'cn=manager',
                       'cn=manager',
                       'cn=manager'],
           'cert_host':['ldap.doesciencegrid.org'],
           'cert_port':['389'],
           'cert_basedn':['ou=People,o=doesciencegrid.org'],
           'cert_bindname':['cn=manager']
          }

  def __init__(self,read=0):
    if len(sys.argv)>1 and sys.argv[0]=='-s':
         self.sfile=sys.argv[1]
    else: self.sfile=self.def_sfile
    try: # we should be able to open and close the settings file
         self.sfile_o=open(self.sfile,"a+")
         self.sfile_o.close()
         rw_ok=1
    except: # try again with def file if another was specified
         if self.sfile!=self.def_sfile:       
           try: # we should be able to open and close the settings file
                self.sfile_o=open(self.sfile,"a+")
                self.sfile_o.close()
                self.rw_ok=1
           except:
             self.sfile_o=None
             self.rw_ok=0
         else:
             self.sfile_o=None
             self.rw_ok=0
    self.written=0

  def rw(self):
     return self.rw_ok
  
  def get_value(self,name):
     if self.options.has_key(name):
       return self.options[name]
     else: return None
  
  def get_all(self):
     return self.options

  def set_value(self,name,value,overwrite=0):
     if self.options.has_key(name) and not overwrite: return 0     
     self.options[name]=value
     self.written=0
     return 1

  # Set values with duplicate testing
  def set_value_dupl(self,names,values,overwrite=1):
     # First test to make sure all the named lists are present
     # and of equal length
     max=0
     min=99999
     dupl=-1
     for i in range(len(names)):
       if not self.options.has_key(names[i]): return 0
       if len(self.options[names[i]])>max: max=len(self.options[names[i]])
       if len(self.options[names[i]])<min: min=len(self.options[names[i]])

     if max!=min or max==0: return 0

     for j in range(len(self.options[names[0]])):
       list=[]
       for i in range(len(names)):     
         list.append(self.options[names[i]][j])
       if list==values:
         dupl=j
         break

     for i in range(len(names)):     
       if dupl>=0 and overwrite:
         self.options[names[i]]=self.options[names[i]][:dupl]+self.options[names[i]][dupl+1:]
       self.options[names[i]]=[values[i]]+self.options[names[i]]
#       sys.stdout.write("%s=%s\n"%(names[i],repr(self.options[names[i]])))
     if dupl==-1:
       self.written=0
     return 1

  def delete(self,name):
     if self.options.has_key[name]:
       del self.options[name]
       self.written=0
       return 1
     else: return 0

  def read(self,filename=sfile):
     if filename==None:
       filename=self.sfile
     self.sfile_o=open(filename,"r")
     try:
       noptions=pickle.load(self.sfile_o)
     except:
#       raise
       try: self.sfile_o.close()
       except: pass
       self.written=0
       return 0
     self.sfile_o.close()
#     self.options=noptions
     for key in self.options.keys():
       if noptions.has_key(key): continue
       else: noptions[key]=self.options[key]
     self.options=noptions
     self.written=1
     return 1

  def write(self,filename=sfile):
     if self.written==1:
       return 1
     if filename==None:
       filename=self.sfile
     try: # Try to rename the possibly existing settings file to create
          # a backup
       os.rename(self.sfile,self.sfile+"~")
     except:
       pass

     try:
       self.sfile_o=open(filename,"w")
       pickler=pickle.dump(self.options,self.sfile_o,)
     except:
       raise
       try: self.sfile_o.close()
       except: 
         raise
         pass
       self.written=0
       return 0
     self.sfile_o.close()
     self.written=1
     return 1   

class connect_wizard:
  settings=None
  main_win=None
  combo=[]
  entry=None
  names=["Host","Port","BaseDN","BindName"]
  opts=["host","port","basedn","bindname"]
  def __init__(self,settings,main_win, create_db=gtk.TRUE):
     self.settings=settings
     self.main_win=main_win
     params={"funcs": [self.connect_params,self.connect_status],
            "finish": self.pre_finish,
            "labels": ["Enter group server connection values","Connection status"],
            "title": "Connection_wizard",
            "mode" :2
            }
     cw=wizard.Wizard(params,title="Caltech VO Group Manager")

  def next_pressed(self,widget,next):
    next.emit("clicked")

  def create_db_toggled(self,toggle):
    if toggle.get_active():
      self.create_db=gtk.TRUE
    else: self.create_db=gtk.FALSE
    self.ou_entry.set_sensitive(self.create_db)

  def connect_params(self,params,page_box,next=None,prev=None,cancel=None):
    frame=gtk.GtkFrame()
    table=gtk.GtkTable(7,2,gtk.FALSE)
    table.set_border_width(3)
    table.set_col_spacings(5)
    table.set_row_spacings(5)

    page_box.pack_start(frame)
    frame.add(table)
    
    self.combo=[]
    for i in range(len(self.opts)):
      self.combo.append(gtk.GtkCombo())
      self.combo[i].set_popdown_strings(self.settings.get_value(self.opts[i]))
      table.attach(justlabel(self.names[i]+":"), 0, 1, i, i+1,yoptions=0,xpadding=5,
                   xoptions=gtk.FILL)
      table.attach(self.combo[i], 1, 2,i,i+1,yoptions=0,xpadding=5,
                   xoptions=gtk.EXPAND|gtk.FILL)

    i=i+1
    self.entry=gtk.GtkEntry()
    self.entry.set_visibility(gtk.FALSE)
    self.entry.connect("activate",self.next_pressed,next)
    table.attach(justlabel("Password:"), 0, 1, i, i+1,yoptions=0,xpadding=5,
                 xoptions=gtk.FILL)
    table.attach(self.entry, 1, 2,i,i+1,yoptions=0,xpadding=5,
                 xoptions=gtk.EXPAND|gtk.FILL)
    i=i+1
    table.attach(gtk.GtkHSeparator(), 0, 2, i, i+1,yoptions=3,xpadding=3,
                 xoptions=gtk.EXPAND|gtk.FILL)
    i=i+1
    table.attach(justlabel("Init LDAP\ndirectory"), 0, 1, i, i+1,yoptions=0,xpadding=5,
                 xoptions=gtk.FILL)
    checkbutton=gtk.GtkCheckButton()
    checkbutton.set_active(gtk.FALSE)
    self.create_db=gtk.FALSE
    checkbutton.connect("toggled",self.create_db_toggled)
    table.attach(checkbutton, 1, 2,i,i+1,yoptions=0,xpadding=5,
                 xoptions=gtk.EXPAND)
    i=i+1
    self.ou_entry=gtk.GtkEntry()
    table.attach(self.ou_entry, 1, 2, i, i+1,yoptions=0,xpadding=5,
                 xoptions=gtk.FILL)
    self.ou_entry.set_sensitive(gtk.FALSE)
    table.attach(justlabel("Org:"), 0, 1,i,i+1,yoptions=0,xpadding=5,
                 xoptions=gtk.FILL)

  def download(self):
    self.new_users={}
    num=0
    num_groups=0
    maxspin=5

    status=""
    r=None
    oldr=None
    spin=0
    starttime=starttime=time.time()
    while num<25:
      num=num+1
      if r: oldr=r[0]
      else: oldr=None
      r=None
      gtk_process_events()
      try:
        r=self.main_win.l.result(self.result_id,timeout=1.0,all=0)
#        if r: print "r not None"
#        else: print "r=None"
#        if oldr: print "oldr not None"
#        else: print "oldr=None"

        if r and not r[0]:
          spin=spin+1
          if not oldr and spin>=maxspin: 
            spin=0
            self.label.set_text("Waiting for data...")
            while gtk.events_pending():
              gtk.mainiteration(block=gtk.FALSE)
#            print "Sleeping"
            time.sleep(1)
#          continue
        else:
          num=0 # Reset num when results are received
      except ldap.TIMEOUT,msg:
        status="Error "+str(msg)
        raise
      except ldap.TIMELIMIT_EXCEEDED,msg:
        status="Error "+str(msg)
        rasie
      except Exception,msg:
        status="Error "+str(msg)
        raise
      if r:
        if r[1]: 
          status="Done"
          for j in xrange(len(r[1])):
            user=r[1][j]
            if user: self.new_groups[user[0]]=user[1]
          endtime=time.time()
          if endtime-starttime>0.5 or num_groups==0 or self.label.get_text()=='Waiting for data...':
            self.label.set_text(user[0])
            starttime=time.time()
          while gtk.events_pending():
            gtk.mainiteration(block=gtk.FALSE)
        else:
          continue
    if status=="": status='Done'
    self.label.set_text(status)
    gtk_process_events()
    return self.new_groups

  def connect_status(self,params,page_box,next=None,prev=None,cancel=None):
    frame=gtk.GtkFrame()
    table=gtk.GtkTable(2,7,gtk.FALSE)
    table.set_border_width(3)
    table.set_col_spacings(5)
    table.set_row_spacings(5)
    next.set_sensitive(gtk.FALSE)
    gtk_process_events()
    page_box.pack_start(frame)
    frame.add(table)
    frame.show_all()
    self.settings.write()

# First open the group database
    lopts=[]
    i=1
    for i in range(len(self.opts)):
      lopts.append(self.combo[i].entry.get_text())
    host=lopts[0]
    port=lopts[1]
    basedn=lopts[2]
    bindname=lopts[3]
    self.settings.set_value_dupl(self.opts,lopts)
    self.settings.write()
    passwd=self.entry.get_text()

    table.attach(justlabel("Connecting to:"), 
                 0, 1, i, i+1,yoptions=0,xpadding=5,
                   xoptions=gtk.EXPAND|gtk.FILL)
    table.attach(justlabel("ldap://%s:%s"%(host,port)), 1, 2,i,i+1,
                   yoptions=0,xpadding=5,xoptions=gtk.EXPAND|gtk.FILL)
    frame.show_all()

    while gtk.events_pending():
      gtk.mainiteration(block=gtk.FALSE)
    l=None
    try:
      l=self.main_win.ldap_init_group(host=host,port=int(port),
                basedn=basedn,bindname=bindname,
                passwd=passwd)
      status="Connected"
      self.settings.set_value_dupl(self.opts,lopts)
    except ldap.LDAPError,v:
      status="Error: "+v[0]['desc']
    except Exception,v:
      status="Error: "+v[0]

    i=i+1
    table.attach(justlabel("Status:"), 
                 0, 1, i, i+1,yoptions=0,xpadding=5,
                   xoptions=gtk.FILL|gtk.EXPAND)
    table.attach(justlabel(status), 1, 2,i,i+1,yoptions=0,xpadding=5,
                   xoptions=gtk.EXPAND|gtk.FILL)
                   
    table.show_all()
    gtk_process_events()

# Optionally initialize the group database
    if self.create_db:
      i=i+1
      table.attach(justlabel("Creating DB:"), 
                 0, 1, i, i+1,yoptions=0,xpadding=5,
                   xoptions=gtk.EXPAND|gtk.FILL)
      gtk_process_events()
      org=self.ou_entry.get_text()
      dnparts=basedn.split(",")
      dc=None
      for part in dnparts:
        p1=part.split("=")
        map(string.strip,p1)
        if (p1[0]=='dc' or p1[0]=='o') and len(p1)==2:
          dc=p1[1]
          break
      if not dc:
        table.attach(justlabel("Can't find 'dc' or 'o' elements in basedn"), 1, 2,i,i+1,yoptions=0,xpadding=5,
                     xoptions=gtk.EXPAND|gtk.FILL)
        next.set_sensitive(gtk.FALSE)
        table.show_all()    
        return

      org_created=0
      man_created=0
      try:
        org_obj=[("objectclass",["dcObject","organization"]),
                 ("o",["%s"%org]),
                 ("dc",dc)]
        l.add_s("%s"%basedn,org_obj)
        org_created=1
        status='OK'
      except ldap.ALREADY_EXISTS,v:
        org_created=1
        pass
      except ldap.LDAPError,v:
          status="Error creating organization: "+v[0]['desc']
          pass
      except Exception,v:
          status="Error creating organization:"+v[0]
          pass
      if org_created:
        try:
          manager=bindname.split("=")
          map(string.strip,manager)
          if len(manager)==2 and manager[0]=="cn":
            manager_name=manager[1].capitalize()
          elif len(manager)==1:
            manager_name=manager[0].capitalize()
          else: manager_name="Manager"


          l.add_s("cn=%s,%s"%(manager_name.lower(),basedn),
                [("objectclass",["organizationalRole"]),
                 ("cn",manager_name),
                 ("description",["Directory Manager"])])
          man_created=1
          status="OK"
        except ldap.ALREADY_EXISTS,v:
          man_created=1
          pass
        except ldap.LDAPError,v:
            status="Error creating manager: "+v[0]['desc']
            pass
        except Exception,v:
            status="Error creating manager: "+v[0]
            pass
        people_created=0
        try:
          l.add_s("ou=People,%s"%basedn,
                [("objectclass",['top', 'domain', 'organizationalUnit']),
                 ("ou",['People']),
                 ("dc",['People'])])
          people_created=1
          status="OK"
        except ldap.ALREADY_EXISTS,v:
          people_created=1
          pass
        except ldap.LDAPError,v:
          status="Error creating 'People' tree: "+v[0]['desc']
          pass
        except Exception,v:
          status="Error creating 'People' tree: "+v[0]
          pass

      if org_created and man_created:
        status="OK"
      table.attach(justlabel(status), 1, 2,i,i+1,yoptions=0,xpadding=5,
                     xoptions=gtk.EXPAND|gtk.FILL)
      gtk_process_events()
      if not (org_created and man_created and people_created):
        next.set_sensitive(gtk.FALSE)
        return

    gtk_process_events()

#    if self.main_win.init_panes(): 
#       status="Group info downloaded"
#    else: status="Error"     
    i=i+1
    self.label=justlabel('Sending request...')
    table.attach(justlabel("Downloading\ngroup info:"), 0, 1,i,i+1,yoptions=0,xpadding=5,
                   xoptions=gtk.EXPAND|gtk.FILL)

    table.attach(self.label, 1, 2,i,i+1,yoptions=0,xpadding=5,
                   xoptions=gtk.EXPAND|gtk.FILL)
    self.result_id=self.main_win.request_groups(self.main_win.l,root=self.main_win.basedn)
    self.new_groups={}
    table.show_all()
    gtk_process_events()
    
    self.download()
    self.main_win.users_downloaded=0
    self.main_win.init_panes(self.new_groups)
    self.settings.set_value_dupl(self.opts,lopts)
    i=i+2
# Now open the user database -- deprecated 
#    if self.use_group:
    self.main_win.userldap=self.main_win.l

    table.show_all()    
    next.set_sensitive(gtk.TRUE)

  def pre_finish(self,params):
    pass

class connect_users_wizard:
  settings=None
  main_win=None
  combo=[]
  entry=None
  l=None
  names=["Host","Port","BaseDN","BindName"]
  opts=["cert_host","cert_port","cert_basedn","cert_bindname"]
  def __init__(self,settings,main_win):
     self.settings=settings
     self.main_win=main_win
     params={"funcs": [self.connect_params,self.connect_status,self.save_status],
            "finish": self.pre_finish,
            "labels": ["Enter user server connection values","Download status","Save status"],
            "title": "Connection_wizard",
            "mode" :2
            }
     cw=wizard.Wizard(params,title="Caltech VO Group Manager")

  def connected(self,table,next):
    i=0
    if not self.main_win.l:
      table.attach(gtk.GtkLabel("Not connected!\n\nPlease connect to the LDAP server first"),
                   0, 3, i, i+1,xpadding=5,ypadding=5,
                   xoptions=gtk.FILL|gtk.EXPAND,
                   yoptions=gtk.FILL|gtk.EXPAND)
      next.set_sensitive(gtk.FALSE)
      return 0
    return 1

  def next_pressed(self,widget,next):
    next.emit("clicked")

  def update_db_toggled(self,toggle):
    if toggle.get_active():
      self.update_db=gtk.TRUE
    else: self.update_db=gtk.FALSE

  def connect_params(self,params,page_box,next=None,prev=None,cancel=None):
    frame=gtk.GtkFrame()
    table=gtk.GtkTable(7,2,gtk.FALSE)
    table.set_border_width(3)
    table.set_col_spacings(5)
    table.set_row_spacings(5)

    page_box.pack_start(frame)
    frame.add(table)
    if not self.connected(table,next):
      frame.show_all()
      return

    self.combo=[]
    for i in range(len(self.opts)):
      self.combo.append(gtk.GtkCombo())
      self.combo[i].set_popdown_strings(self.settings.get_value(self.opts[i]))
      table.attach(justlabel(self.names[i]+":"), 0, 1, i, i+1,yoptions=0,xpadding=5,
                   xoptions=gtk.FILL)
      table.attach(self.combo[i], 1, 2,i,i+1,yoptions=0,xpadding=5,
                   xoptions=gtk.EXPAND|gtk.FILL)

    i=i+1
    self.entry=gtk.GtkEntry()
    self.entry.set_visibility(gtk.FALSE)
    self.entry.connect("activate",self.next_pressed,next)
    table.attach(justlabel("Password:"), 0, 1, i, i+1,yoptions=0,xpadding=5,
                 xoptions=gtk.FILL)
    table.attach(self.entry, 1, 2,i,i+1,yoptions=0,xpadding=5,
                 xoptions=gtk.EXPAND|gtk.FILL)
    i=i+1
    table.attach(gtk.GtkHSeparator(), 0, 2, i, i+1,yoptions=3,xpadding=3,
                 xoptions=gtk.EXPAND|gtk.FILL)
    i=i+1
    table.attach(justlabel("Update group DB with newer users?"), 0, 1, i, i+1,yoptions=0,xpadding=5,
                 xoptions=gtk.FILL)
    checkbutton=gtk.GtkCheckButton()
    checkbutton.set_active(gtk.FALSE)
    self.create_db=gtk.FALSE
    checkbutton.connect("toggled",self.update_db_toggled)
    table.attach(checkbutton, 1, 2,i,i+1,yoptions=0,xpadding=5,
                 xoptions=gtk.EXPAND)
    i=i+1
#    self.ou_entry=gtk.GtkEntry()
#    table.attach(self.ou_entry, 1, 2, i, i+1,yoptions=0,xpadding=5,
#                 xoptions=gtk.FILL)
#    self.ou_entry.set_sensitive(gtk.FALSE)
#    table.attach(justlabel("Org:"), 0, 1,i,i+1,yoptions=0,xpadding=5,
#                 xoptions=gtk.FILL)

  def connect_status(self,params,page_box,next=None,prev=None,cancel=None):
    frame=gtk.GtkFrame()
    table=gtk.GtkTable(2,7,gtk.FALSE)
    table.set_border_width(3)
    table.set_col_spacings(5)
    table.set_row_spacings(5)
    next.set_sensitive(gtk.TRUE)
    page_box.pack_start(frame)
    frame.add(table)
    frame.show_all()
    self.settings.write()
# First open the group database
    lopts=[]
    i=1
    for i in range(len(self.opts)):
      lopts.append(self.combo[i].entry.get_text())
    host=lopts[0]
    port=lopts[1]
    basedn=lopts[2]
    bindname=lopts[3]

    self.settings.set_value_dupl(self.opts,lopts)
    self.settings.write()
    passwd=self.entry.get_text()

    table.attach(justlabel("Connecting to:"), 
                 0, 1, i, i+1,yoptions=0,xpadding=5,
                   xoptions=gtk.EXPAND|gtk.FILL)
    table.attach(justlabel("ldap://%s:%s"%(host,port)), 1, 2,i,i+1,
                   yoptions=0,xpadding=5,xoptions=gtk.EXPAND|gtk.FILL)
    frame.show_all()
    gtk_process_events()
    l=None
    status=""
    try:
      l=self.main_win.ldap_init(host=host,port=int(port),
                basedn=basedn,bindname=bindname,
                passwd="")
      status="Connected"

      self.settings.set_value_dupl(self.opts,lopts)
    except ldap.LDAPError,v:
      print v
      status="Error: "+v[0]['desc']
    except Exception,v:
      print v
      status="Error: "+str(v[0])
    self.l=l

    i=i+1
    table.attach(justlabel("Status:"), 
                 0, 1, i, i+1,yoptions=0,xpadding=5,
                   xoptions=gtk.FILL|gtk.EXPAND)
    table.attach(justlabel(status), 1, 2,i,i+1,yoptions=0,xpadding=5,
                   xoptions=gtk.EXPAND|gtk.FILL)
                   
    table.show_all()
    gtk.mainiteration(block=gtk.FALSE)

    result_id=self.l.search(basedn,ldap.SCOPE_SUBTREE,"usercertificate;binary=*")

    self.new_users={}

    num=0
    maxspin=5
    label=justlabel("--")
    i=i+1
    table.attach(label, 1, 2,i,i+1,yoptions=0,xpadding=5,
                   xoptions=gtk.EXPAND|gtk.FILL)
    label.show()
    next.set_sensitive(gtk.FALSE)
    r=None
    oldr=None
    spin=0
    starttime=starttime=time.time()
    while num<25:
      num=num+1
      if r: oldr=r[0]
      else: oldr=None
      r=None
      gtk_process_events()
      try:
        r=l.result(result_id,timeout=1.0,all=0)

        if not r: # This shouldn't happen!
          break
        if r and not r[0]:
          spin=spin+1
          if not oldr and spin>=maxspin: 
            spin=0
            label.set_text("Waiting for data...")
            while gtk.events_pending():
              gtk.mainiteration(block=gtk.FALSE)

            time.sleep(1)
        else:
          num=0 # Reset num when results are received
      except ldap.TIMEOUT,msg:
        status="Error "+str(msg)
        pass
      except ldap.TIMELIMIT_EXCEEDED,msg:
        status="Error "+str(msg)
        pass
      except Exception,msg:
        status="Error "+str(msg)
      if r:
        if r[1]: 
          status="User info downloaded"
          for j in xrange(len(r[1])):
            user=r[1][j]
            if user: self.new_users[user[0]]=user[1]
          while gtk.events_pending():
            gtk.mainiteration(block=gtk.FALSE)

          endtime=time.time()
          if endtime-starttime>1:
            label.set_text(user[0])
            while gtk.events_pending():
              gtk.mainiteration(block=gtk.FALSE)
            starttime=time.time()
        elif r[0]=='RES_SEARCH_RESULT':
          break


# Load user info from the group server if it wasn't done before
#    users=self.main_win.get_users()

    label.set_text(status)
# Now open the user database -- deprecated 
#    if self.use_group:
    self.main_win.userldap=self.main_win.l

    table.show_all()    
    next.set_sensitive(gtk.TRUE)


  def save_status(self,params,page_box,next=None,prev=None,cancel=None):
    frame=gtk.GtkFrame()
    page_box.pack_start(frame)

    next.set_sensitive(gtk.FALSE)
    table=gtk.GtkTable(7,2,gtk.FALSE)
    table.set_border_width(3)
    table.set_col_spacings(5)
    table.set_row_spacings(5)
    frame.add(table)

    i=0
    table.attach(justlabel("Connecting to:"), 
                 0, 1, i, i+1,yoptions=0,xpadding=5,
                   xoptions=gtk.EXPAND|gtk.FILL)
    table.attach(justlabel("ldap://%s:%s"%(self.main_win.host,self.main_win.port)),
                 1, 2,i,i+1,yoptions=0,xpadding=5,
                   xoptions=gtk.EXPAND|gtk.FILL)
    i=i+1
    label=justlabel('--')
    table.attach(justlabel("Status:"), 
                 0, 1, i, i+1,yoptions=0,xpadding=5,
                   xoptions=gtk.EXPAND|gtk.FILL)
    table.attach(label, 1, 2,i,i+1,yoptions=0,xpadding=5,
                   xoptions=gtk.EXPAND|gtk.FILL)
    frame.show_all()
    gtk_process_events()

    table.show_all()
    try:
      starttime=time.time()
      for dn in self.new_users.keys():
        mydn="cn=%s,ou=People,%s"%(self.new_users[dn]['cn'][0],self.main_win.basedn)
        endtime=time.time()
        if endtime-starttime>0.2:
           label.set_text(dn)
           gtk_process_events()
           starttime=time.time()
        if self.main_win.users.has_key(mydn):
          continue
          try:
            self.main_win.l.delete_s(mydn)
          except ldap.NO_SUCH_OBJECT:
            pass
          
          gtk_process_events()

        inlist=['objectclass','cn','sn','description','ou','o']
        dict={}
        
        for el in inlist:
          if self.new_users[dn].has_key(el):
            dict[el]=self.new_users[dn][el]
          if self.new_users[dn].has_key(string.upper(el)):
            dict[el]=self.new_users[dn][string.upper(el)]

        dict['objectclass']=["person","organizationalPerson","inetOrgPerson","pkiUser"]
        dict['ou']=["People"]
        dict['description']='subject= %s'%dn
        if not dict.has_key('sn'):
          dict['sn']=guess_sn(dict['cn'][0])[1]
        self.main_win.users[mydn]=dict
        ltuple=map(lambda x,dict=dict:(x,dict[x]),dict.keys())

        try:
          self.main_win.l.add_s(mydn,ltuple)
        except ldap.ALREADY_EXISTS:
          pass
        gtk_process_events()
        

      status="User info updated"
    except ldap.LDAPError,v:
      status="Error: %s: %s"%(v[0]['desc'],v[0]['info'])
      print v
      traceback.print_tb(sys.exc_traceback)

    except Exception,v:
      status="Error: "+v[0]

      traceback.print_tb(sys.exc_traceback)

    label.set_text(status)
    next.set_sensitive(gtk.TRUE)
    gtk_process_events()
    frame.show_all()


  def pre_finish(self,params):
    pass
 

class group_edit:
  settings=None
  main_win=None
  glist=None
  mlist=None
  names=["Host","Port","BaseDN","BindName"]
  opts=["host","port","basedn","bindname"]
  group_changed=0
  group_selected=None
  
  def __init__(self,settings,main_win):
     self.settings=settings
     self.main_win=main_win
     params={"funcs": [self.select_group,self.edit_group,self.connect_status],
            "finish": self.pre_finish,
            "labels": ["Select group","Select group members", "Update status"],
            "title": "Edit group",
            "info" : "Select group to edit:",
            "mode" :2
            }
     self.group_changed=0
     ge=wizard.Wizard(params,title="Caltech VO Group Manager")
  
  def connected(self,table,next):
    i=0
    if not self.main_win.l:
      table.attach(gtk.GtkLabel("Not connected!\n\nPlease connect to the LDAP server first"),
                   0, 3, i, i+1,xpadding=5,ypadding=5,
                   xoptions=gtk.FILL|gtk.EXPAND,
                   yoptions=gtk.FILL|gtk.EXPAND)
      next.set_sensitive(gtk.FALSE)
      return 0
    return 1

  def select_row(self,*args):
    self.group_selected=self.glist.get_text(args[1],0)

  def unselect_row(self,*args):
    self.group_selected=None

  def select_group(self,params,page_box,next=None,prev=None,cancel=None):
    frame=gtk.GtkFrame()
    table=gtk.GtkTable(1,2,gtk.FALSE)
    table.set_border_width(3)
    table.set_col_spacings(5)
    table.set_row_spacings(5)

    page_box.pack_start(frame)
    frame.add(table)
    if not self.connected(table,next):
      frame.show_all()
      return

    self.glist=gtk.GtkCList(cols=2,titles=["Group name","Description"])
    self.glist.set_column_auto_resize(0,gtk.TRUE)
    self.glist.set_column_auto_resize(1,gtk.TRUE)
    self.glist.set_auto_sort(gtk.TRUE)
    self.glist.connect("select-row",self.select_row)
    self.glist.connect("unselect-row",self.unselect_row)
    i=0
    swin=gtk.GtkScrolledWindow()
    swin.set_policy(GTK.POLICY_NEVER,GTK.POLICY_AUTOMATIC)
    table.attach(gtk.GtkLabel(params['info']), 
                   0, 2, i, i+1,xpadding=5,ypadding=5,
                   xoptions=gtk.FILL|gtk.EXPAND,
                   yoptions=0)
    i=i+1
    table.attach(swin, 0, 1, i, i+1,xpadding=0,
                   xoptions=gtk.FILL|gtk.EXPAND,
                   yoptions=gtk.FILL|gtk.EXPAND)
    swin.add_with_viewport(self.glist)
    keys=self.main_win.members.keys()
    for i in range(len(keys)):
      self.glist.append([self.main_win.members[keys[i]]['dc'][0],
                    self.main_win.members[keys[i]]['cn'][0]])
    rows=self.main_win.glist.selection
    if len(rows)==1:
      self.group_selected=self.main_win.glist.get_text(rows[0],0)
      self.glist.select_row(rows[0],0)
    frame.show_all()
    
  def invert_list_selection(self,clist,col):
    if len(clist.selection)<clist.rows:
      clist.select_all()
    else:
      clist.unselect_all()

  def guess_sn(self,sntuple):
    sn=sntuple[1]
    snlist=string.split(sn,' ')
    if len(snlist)<2: return sntuple[0],sn,sn
    for i in range(len(snlist)-1,-1,-1):
      try:
        real_sn=snlist[i]
        j=int(real_sn)
      except:
        if real_sn[0] not in string.punctuation:
          return sntuple[0],sn,real_sn
        pass
    return sntuple[0],sn,snlist[-1]
    
  def order_by_sn(self,userdict):
    snlist=map(lambda x,mdict=userdict:(x,mdict[x]['cn'][0]),userdict.keys())
    return map(self.guess_sn,snlist)

  def edit_group(self,params,page_box,next=None,prev=None,cancel=None):
    frame=gtk.GtkFrame()
    table=gtk.GtkTable(3,2,gtk.FALSE)
    table.set_border_width(3)
    table.set_col_spacings(5)
    table.set_row_spacings(5)

    page_box.pack_start(frame)
    frame.add(table)
    
    i=0
    if not self.group_selected:
      table.attach(gtk.GtkLabel("No group selected!\n\nPlease go back and click on a group name"), 
                   0, 3, i, i+1,xpadding=5,ypadding=5,
                   xoptions=gtk.FILL|gtk.EXPAND,
                   yoptions=gtk.FILL|gtk.EXPAND)
      next.set_sensitive(gtk.FALSE)
      frame.show_all()
      return

    table.attach(gtk.GtkLabel("Members of group '%s':"%self.group_selected), 
                   0, 1, i, i+1,xpadding=5,ypadding=5,
                   xoptions=gtk.FILL|gtk.EXPAND,
                   yoptions=0)
    table.attach(gtk.GtkLabel("Available users:"), 
                   2, 3, i, i+1,xpadding=5,ypadding=5,
                   xoptions=gtk.FILL|gtk.EXPAND,
                   yoptions=0)

    i=1
    self.inlist=gtk.GtkCList(cols=3,titles=["Last name","Name","Organization"])
    self.inlist.set_column_auto_resize(0,gtk.TRUE)
    self.inlist.set_column_auto_resize(1,gtk.TRUE)
    self.inlist.set_auto_sort(gtk.TRUE)
    self.inlist.set_selection_mode(GTK.SELECTION_EXTENDED)
    self.inlist.connect("click-column",self.invert_list_selection)
    found=0
    for key in self.main_win.members.keys():
     if self.main_win.members[key]['dc'][0]==self.group_selected:
       found=1
       break
    if found: 
      self.members=self.main_win.members[key]['member']
      self.groupkey=key
    swin=gtk.GtkScrolledWindow()
    swin.set_policy(GTK.POLICY_NEVER,GTK.POLICY_AUTOMATIC)

    table.attach(swin, 0, 1, i, i+1,xpadding=5,
                   xoptions=gtk.FILL|gtk.EXPAND,
                   yoptions=gtk.FILL|gtk.EXPAND)
    swin.add_with_viewport(self.inlist)

    self.users=self.main_win.get_users()
    self.ulist=gtk.GtkCList(cols=3,titles=["Last name","Full name","Organization"])
    self.ulist.set_column_auto_resize(0,gtk.TRUE)
    self.ulist.set_column_auto_resize(1,gtk.TRUE)
    self.ulist.set_selection_mode(GTK.SELECTION_EXTENDED)
    self.ulist.connect("click-column",self.invert_list_selection)
    swin=gtk.GtkScrolledWindow()
    swin.set_policy(GTK.POLICY_NEVER,GTK.POLICY_AUTOMATIC)

    table.attach(swin, 2, 3, i, i+1,xpadding=5,
                   xoptions=gtk.FILL|gtk.EXPAND,
                   yoptions=gtk.FILL|gtk.EXPAND)
    swin.add_with_viewport(self.ulist)

#Order the list of users in the group, and those not in the group
    snlist=self.order_by_sn(self.users)
    memdict={}
    for mem in self.members:
      try:
        memdict[mem]=self.users[mem]
      except KeyError: # The group manager is not in the user list
        pass
    memlist=self.order_by_sn(memdict)
# Now fill the two clists:
    self.inlist.freeze()
    self.inlist.set_auto_sort(gtk.TRUE)
    self.inlist.set_sort_column(0)

    for name in memlist:
      mem=name[0]
      try:
        if self.users[mem].has_key('o'):
          row=self.inlist.append([name[2],name[1],
                            string.join(self.users[mem]['o'],", ")])
        else:
          row=self.inlist.append([name[2],name[1],''])
        self.inlist.set_row_data(row,mem)
      except KeyError: # This happens because the manager is a member but not a user
        mseq=map(lambda x:tuple(string.split(x,'=')),ldap.explode_dn(mem))
        dndict=dict(mseq)
        if dndict.has_key('cn') and string.lower(dndict['cn'])=='manager':
          mdn="cn=manager,%s"%self.main_win.basedn
          mdict=dict(map(lambda x:tuple(string.split(x,'=')),ldap.explode_dn(mdn)))
          equal=1
          for key in mdict.keys():
            if not dndict.has_key(key) or string.lower(mdict[key])!=string.lower(dndict[key]):
              equal=0
              break
          if equal:
            row=self.inlist.append([self.main_win.members[self.groupkey]['owner'][0],
                                   "Group manager"])
            self.inlist.set_row_data(row,self.main_win.members[self.groupkey]['owner'][0])
    self.inlist.thaw()

    self.ulist.freeze()
    self.ulist.set_auto_sort(gtk.TRUE)
    self.ulist.set_sort_column(0)
    for name in snlist:
      mem=name[0]
      if not mem in self.members:
        if self.users[mem].has_key('o'):
          row=self.ulist.append([name[2],name[1],
                                 string.join(self.users[mem]['o'],', ')])
        else:
          row=self.ulist.append([name[2],name[1],''])
        self.ulist.set_row_data(row,mem)
    self.ulist.thaw()

# Add the add and remove buttons
    vbox=gtk.GtkVBox(gtk.TRUE)
    self.add_button=gtk.GtkButton("< Add")
    self.rem_button=gtk.GtkButton("Remove >")
    vbox.pack_start(self.add_button,expand=gtk.FALSE,fill=gtk.FALSE)
    vbox.pack_start(self.rem_button,expand=gtk.FALSE,fill=gtk.FALSE)
    table.attach(vbox, 1, 2, i, i+1,xpadding=5,
                   xoptions=0,
                   yoptions=gtk.FILL|gtk.EXPAND)
    self.add_button.connect("pressed",self.user_added)
    self.rem_button.connect("pressed",self.user_deleted)
    frame.show_all()

  def user_added(self,button):
    sel=self.ulist.selection
    self.ulist.freeze()
    while len(sel)>0:
      row=sel[0]
      data=self.ulist.get_row_data(row)
      text=[self.ulist.get_text(row,0),
            self.ulist.get_text(row,1),
            self.ulist.get_text(row,2)]
      self.ulist.remove(row)
      nrow=self.inlist.append(text)
      self.inlist.set_row_data(nrow,data)
      sel=self.ulist.selection
    self.ulist.thaw()
    self.group_changed=1

  def user_deleted(self,button):
    sel=self.inlist.selection
    self.inlist.freeze()
    while len(sel)>0:
      row=sel[0]
      data=self.inlist.get_row_data(row)
      text=[self.inlist.get_text(row,0),
            self.inlist.get_text(row,1),
            self.inlist.get_text(row,2)]
      self.inlist.remove(row)
      nrow=self.ulist.append(text)
      self.ulist.set_row_data(nrow,data)
      sel=self.inlist.selection
    self.inlist.thaw()
    self.group_changed=1


  def connect_status(self,params,page_box,next=None,prev=None,cancel=None):
    frame=gtk.GtkFrame()
    page_box.pack_start(frame)
    members=[]
    for row in range(self.inlist.rows):
      members.append(self.inlist.get_row_data(row))
    if len(members)==0:
      members=self.main_win.members[self.groupkey]['owner']
    self.main_win.members[self.groupkey]['member']=members
    ldap_list=[]
    for key in self.main_win.members[self.groupkey].keys():
      ldap_list.append((ldap.MOD_REPLACE,key,self.main_win.members[self.groupkey][key]))

    table=gtk.GtkTable(7,2,gtk.FALSE)
    table.set_border_width(3)
    table.set_col_spacings(5)
    table.set_row_spacings(5)
    frame.add(table)

    i=0
    if self.group_changed==1:
      table.attach(justlabel("Connecting to:"), 
                   0, 1, i, i+1,yoptions=0,xpadding=5,
                     xoptions=gtk.EXPAND|gtk.FILL)
      table.attach(justlabel("ldap://%s:%s"%(self.main_win.host,self.main_win.port)),
                   1, 2,i,i+1,yoptions=0,xpadding=5,
                     xoptions=gtk.EXPAND|gtk.FILL)
      table.show_all()    
      try:
        result=self.main_win.l.modify(self.groupkey,ldap_list)
        gtk_process_events()

        a=self.main_win.l.result(result)
        status="Group info updated"
      except ldap.LDAPError,v:
        status="Error: "+v[0]['desc']
        pass
      except Exception,v:
        status="Error: "+v[0]
        pass
      i=i+1
      table.attach(justlabel("Status:"), 
                   0, 1, i, i+1,yoptions=0,xpadding=5,
                     xoptions=gtk.EXPAND|gtk.FILL)
      table.attach(justlabel(status), 1, 2,i,i+1,yoptions=0,xpadding=5,
                     xoptions=gtk.EXPAND|gtk.FILL)
      gtk_process_events()
    else:
      table.attach(gtk.GtkLabel("No update needed"), 
                   0, 2, i, i+1,
                     yoptions=gtk.EXPAND|gtk.FILL,
                     xpadding=5,
                     xoptions=gtk.EXPAND|gtk.FILL)
      table.show_all()    

    table.show_all()    
    frame.show_all()

  def pre_finish(self,params):
   if self.group_changed:
     self.main_win.fill_group_list()
     self.main_win.fill_members_list_cb()
   pass


class delete_group(group_edit):
  def __init__(self,settings,main_win):
     self.settings=settings
     self.main_win=main_win
     params={"funcs": [self.select_group,self.delete_status],
            "finish": self.pre_finish,
            "labels": ["Select group","Update status"],
            "title": "Edit group",
            "info" : "Select group to delete:",
            "mode" :2
            }
     self.group_changed=0
     gd=wizard.Wizard(params,title="Caltech VO Group Manager")

  def delete_status(self,params,page_box,next=None,prev=None,cancel=None):
    frame=gtk.GtkFrame()
    table=gtk.GtkTable(3,2,gtk.FALSE)
    table.set_border_width(3)
    table.set_col_spacings(5)
    table.set_row_spacings(5)

    page_box.pack_start(frame)
    frame.add(table)
    
    i=0
    if not self.group_selected:
      table.attach(gtk.GtkLabel("No group selected!\n\nPlease go back and click on a group name"), 
                   0, 3, i, i+1,xpadding=5,ypadding=5,
                   xoptions=gtk.FILL|gtk.EXPAND,
                   yoptions=gtk.FILL|gtk.EXPAND)
      next.set_sensitive(gtk.FALSE)
      frame.show_all()
      return


    i=i+1

    table.attach(justlabel("Connecting to:"), 
                 0, 1, i, i+1,yoptions=0,xpadding=5,
                   xoptions=gtk.EXPAND|gtk.FILL)
    table.attach(justlabel("ldap://%s:%s"%(self.main_win.host,self.main_win.port)),
                 1, 2,i,i+1,yoptions=0,xpadding=5,
                   xoptions=gtk.EXPAND|gtk.FILL)
    found=0
    for key in self.main_win.members.keys():
     if self.main_win.members[key]['dc'][0]==self.group_selected:
       found=1
       break
    if found: 
      self.members=self.main_win.members[key]['member']
      self.groupkey=key
      try:
          result=self.main_win.l.delete(self.groupkey)
          gtk_process_events()

          a=self.main_win.l.result(result)
          del self.main_win.members[self.groupkey]
          status="Group '%s' deleted"%self.group_selected
          self.group_changed=1
      except ldap.LDAPError,v:
          status="Error: "+v[0]['desc']
          pass
      except Exception,v:
          status="Error: "+v[0]
          pass
      i=i+1
      table.attach(justlabel("Status:"), 
                     0, 1, i, i+1,yoptions=0,xpadding=5,
                       xoptions=gtk.EXPAND|gtk.FILL)
      table.attach(justlabel(status), 1, 2,i,i+1,yoptions=0,xpadding=5,
                       xoptions=gtk.EXPAND|gtk.FILL)
      gtk_process_events()
    else:
      status="Hmmm, can't find that group!"

    table.attach(justlabel("Status:"), 
                  0, 1, i, i+1,yoptions=0,xpadding=5,
                       xoptions=gtk.EXPAND|gtk.FILL)
    table.attach(justlabel(status), 1, 2,i,i+1,yoptions=0,xpadding=5,
                 xoptions=gtk.EXPAND|gtk.FILL)

    frame.show_all()

  def pre_finish(self,params):
   if self.group_changed:
     self.main_win.fill_group_list()
     self.main_win.mlist.clear()
   pass


class new_group(group_edit):
  def __init__(self,settings,main_win):
     self.settings=settings
     self.main_win=main_win
     params={"funcs": [self.get_group_info,self.create_status,self.edit_group,
                       self.connect_status],
            "finish": self.pre_finish,
            "labels": ["Provide new group info","Group create status",
                       "Select group members", "Update status"],
            "title": "Edit group",
            "mode" :2
            }
     self.group_changed=0
     ng=wizard.Wizard(params,title="Caltech VO Group Manager")

  def get_group_info(self,params,page_box,next=None,prev=None,cancel=None):
    frame=gtk.GtkFrame()
    page_box.pack_start(frame)

    table=gtk.GtkTable(2,3,gtk.FALSE)
    table.set_border_width(3)
    table.set_col_spacings(5)
    table.set_row_spacings(5)
    frame.add(table)
    if not self.connected(table,next):
      frame.show_all()
      return

    names=["Group name","Short description"]
    self.entries=[]
    for i in range(len(names)):
      table.attach(justlabel(names[i]), 
                   0, 1, i, i+1,yoptions=0,xpadding=3,
                     xoptions=gtk.EXPAND|gtk.FILL)
      self.entries.append(gtk.GtkEntry())
      table.attach(self.entries[i], 1, 2,i,i+1,yoptions=0,xpadding=3,
                     xoptions=gtk.EXPAND|gtk.FILL)

    frame.show_all()

  def create_status(self,params,page_box,next=None,prev=None,cancel=None):
    frame=gtk.GtkFrame()
    page_box.pack_start(frame)

    gname=self.entries[0].get_text().strip()
    gdesc=self.entries[1].get_text().split(",")
    gdesc=map(string.strip,gdesc)

    table=gtk.GtkTable(2,3,gtk.FALSE)
    table.set_border_width(3)
    table.set_col_spacings(5)
    table.set_row_spacings(5)
    frame.add(table)
    i=0
    full_gname="ou=%s,%s"%(gname,self.main_win.basedn)
    table.attach(justlabel("Group name:"), 
                 0, 1, i, i+1,yoptions=0,xpadding=5,
                 xoptions=gtk.EXPAND|gtk.FILL)
    
    table.attach(justlabel(gname), 1, 2,i,i+1,yoptions=0,xpadding=5,
                 xoptions=gtk.EXPAND|gtk.FILL)
    if self.main_win.members.has_key(full_gname):
      i=i+1
      table.attach(gtk.GtkLabel("Error: This group already exists!\n"+
                                "Press 'Back' to select a new group name"), 
                   0, 2, i, i+1,yoptions=0,xpadding=5,ypadding=10,
                   xoptions=gtk.EXPAND|gtk.FILL)
      next.set_sensitive(gtk.FALSE)
      frame.show_all()
      return
    
    tuple_list=[("dc",[gname]),
                ("cn",gdesc),
                ("owner",["%s, %s"%(self.main_win.bindname,self.main_win.basedn)]),
                ("objectclass",["domain","organizationalUnit","groupofnames"]),
                ("ou",[gname]),
                ("member",["%s, %s"%(self.main_win.bindname,self.main_win.basedn)])]
    gdict={"dc":[gname],
           "cn":gdesc,
           "owner":["%s, %s"%(self.main_win.bindname,self.main_win.basedn)],
           "objectclass":["domain","organizationalUnit","groupofnames"],
           "ou":[gname],
           "member":["%s, %s"%(self.main_win.bindname,self.main_win.basedn)]}
                
    try:
      result=self.main_win.l.add(full_gname,tuple_list)
      a=self.main_win.l.result(result)
      status="Group created"
      self.group_selected=gname
      self.group_changed=1
      self.main_win.members[full_gname]=gdict
      error=0
    except ldap.LDAPError,v:
      status="Error: "+v[0]['desc']
      error=1
    except Exception,v:
      status="Error: "+v[0]
      error=1
    i=i+1
    table.attach(justlabel("Status:"), 
                 0, 1, i, i+1,yoptions=0,xpadding=5,
                 xoptions=gtk.EXPAND|gtk.FILL)
    table.attach(justlabel(status), 
                 1, 2, i, i+1,yoptions=0,xpadding=5,
                 xoptions=gtk.EXPAND|gtk.FILL)

    i=i+1
    if not error:
      table.attach(gtk.GtkLabel("Press 'Next' to add users to this group"), 
                 0, 2, i, i+1,yoptions=0,xpadding=5,ypadding=10,
                 xoptions=0)
    else:
      next.set_sensitive(gtk.FALSE)
    frame.show_all()
    
class pem_import:
  settings=None
  main_win=None
  glist=None
  mlist=None
  group_changed=0
  group_selected=None
  
  def __init__(self,settings,main_win):
     self.settings=settings
     self.main_win=main_win
     params={"funcs": [self.select_file,self.verify_info,self.connect_status],
            "finish": self.pre_finish,
            "labels": ["Select file","Verify certificate info", "Update status"],
            "title": "Edit group",
            "info" : "Select group to edit:",
            "mode" :2
            }
     self.group_changed=0
     ge=wizard.Wizard(params,title="Caltech VO Group Manager")

  def connected(self,parent,next):
    if not self.main_win.l:
      label=gtk.GtkLabel("Not connected!\n\nPlease connect to the LDAP server first")
      label.set_padding(10,10)
      parent.add(label)
      next.set_sensitive(gtk.FALSE)
      return 0
    return 1

  def crypto_available(self,parent,next):
    global M2Crypto
    if not M2Crypto:
      label=gtk.GtkLabel("M2Crypto module not loaded!")
      label.set_padding(10,10)
      parent.add(label)
      next.set_sensitive(gtk.FALSE)
      return 0
    return 1


  def select_file(self,params,page_box,next=None,prev=None,cancel=None):
    global M2Crypto
    frame=gtk.GtkFrame()
    page_box.pack_start(frame)
    if not self.connected(frame,next) or not self.crypto_available(frame,next):
      frame.show_all()
      return

    fs=gtk.GtkFileSelection()
#    if fs.help_button: fs.help_button.destroy()
    fs.ok_button.destroy()
    fs.cancel_button.destroy()
    fs.main_vbox.reparent(frame)
    fs.main_vbox.set_border_width(5)
    fs.hide_fileop_buttons()
    fs.set_filename(self.main_win.get_cwd())
    self.fs=fs
    frame.show()

  def get_dict(self,elems):
      dict={}
      for u in string.split(elems,"/"):
        e=string.split(u,"=")
        if len(e)==2:
          key=string.lower(e[0])
          if dict.has_key(key): 
           dict[key].append(e[1])
          else:
           dict[key]=[e[1]]
      return dict

  def verify_info(self,params,page_box,next=None,prev=None,cancel=None):
    frame=gtk.GtkFrame()
    table=gtk.GtkTable(1,2,gtk.FALSE)
    table.set_border_width(3)
    table.set_col_spacings(5)
    table.set_row_spacings(5)
    self.filename=self.fs.get_filename()
    page_box.pack_start(frame)
    frame.add(table)
    i=0
    table.attach(justlabel("File selected:"), 
                 0, 1, i, i+1,yoptions=0,xpadding=5,
                 xoptions=gtk.EXPAND|gtk.FILL)
    table.attach(justlabel(self.filename), 1, 2,i,i+1,yoptions=0,xpadding=5,
                 xoptions=gtk.EXPAND|gtk.FILL)
    i=i+1
    try:
      user_cert=X509.load_cert(self.filename)
    except ValueError:
      table.attach(justlabel("Error:"), 
                   0, 1, i, i+1,yoptions=0,xpadding=5,
                   xoptions=gtk.EXPAND|gtk.FILL)
      table.attach(justlabel("Cannot import file\n'%s'\nMake sure this is a public key in PEM format"%self.filename),
                    1, 2,i,i+1,yoptions=0,xpadding=5,
                   xoptions=gtk.EXPAND|gtk.FILL)
      frame.show_all()
      return
    except IOError:
      table.attach(justlabel("Error:"), 
                   0, 1, i, i+1,yoptions=0,xpadding=5,
                   xoptions=gtk.EXPAND|gtk.FILL)
      table.attach(justlabel("Cannot open file named\n'%s'"%self.filename),
                    1, 2,i,i+1,yoptions=0,xpadding=5,
                   xoptions=gtk.EXPAND|gtk.FILL)
      frame.show_all()
      return
    except Exception,v:
      table.attach(justlabel("Error:"), 
                   0, 1, i, i+1,yoptions=0,xpadding=5,
                   xoptions=gtk.EXPAND|gtk.FILL)
      A=""+Err.get_error_reason(Err.get_error_code())
      table.attach(justlabel("Unknown: "+str(A)),
                    1, 2,i,i+1,yoptions=0,xpadding=5,
                   xoptions=gtk.EXPAND|gtk.FILL)
      frame.show_all()
      return

    i=i+1
    table.attach(justlabel("Cetificate info:"), 
                   0, 1, i, i+1,yoptions=0,xpadding=5,
                   xoptions=gtk.EXPAND|gtk.FILL)

    bin_cert=user_cert.as_der()
    elems=str(user_cert.get_subject())
    dict=self.get_dict(elems)
    if dict.has_key('dc'):
      del dict['dc']

    for key in dict.keys():
      for elem in dict[key]:
        i=i+1
        table.attach(justlabel(key+"=",xalign=1), 
                     0, 1, i, i+1,yoptions=0,xpadding=5,
                     xoptions=gtk.EXPAND|gtk.FILL)
        table.attach(justlabel(elem),
                      1, 2,i,i+1,yoptions=0,xpadding=5,
                     xoptions=gtk.EXPAND|gtk.FILL)
    if not dict.has_key('sn'): 
      name=string.split(dict['cn'][0]," ")
      dict['sn']=guess_sn(dict['cn'][0])[1]

    dict['objectclass']=["person","organizationalPerson","inetOrgPerson","pkiUser"]
    dict['ou']=["People"]
    dict['description']='subject= %s'%dn

#    dict['userCertificate;binary']=[user_cert.as_der()]     
    self.tuples=[]
    for key in dict.keys():
     self.tuples.append((key,dict[key]))
    self.cert_dict=dict
    
    i=i+1
    table.attach(justlabel("Issued by:"), 
                   0, 1, i, i+1,yoptions=0,xpadding=5,
                   xoptions=gtk.EXPAND|gtk.FILL)
    elems=str(user_cert.get_issuer())
    dict=self.get_dict(elems)

    for key in dict.keys():
      for elem in dict[key]:
        i=i+1
        table.attach(justlabel(key+"=",xalign=1), 
                     0, 1, i, i+1,yoptions=0,xpadding=5,
                     xoptions=gtk.EXPAND|gtk.FILL)
        table.attach(justlabel(elem),
                      1, 2,i,i+1,yoptions=0,xpadding=5,
                     xoptions=gtk.EXPAND|gtk.FILL)
    verify_status=user_cert.verify()

    if not verify_status: verify_string="OK"
    else: verify_string="Fail!"

    i=i+1
    table.attach(justlabel("Verification:"), 
                 0, 1, i, i+1,yoptions=0,xpadding=5,
                 xoptions=gtk.EXPAND|gtk.FILL)
    table.attach(justlabel(verify_string),
                  1, 2,i,i+1,yoptions=0,xpadding=5,
                 xoptions=gtk.EXPAND|gtk.FILL)
    dates=[str(user_cert.get_not_before()),str(user_cert.get_not_after())]

    i=i+1
    table.attach(justlabel("Not valid before:"), 
                 0, 1, i, i+1,yoptions=0,xpadding=5,
                 xoptions=gtk.EXPAND|gtk.FILL)
    table.attach(justlabel(dates[0]),
                  1, 2,i,i+1,yoptions=0,xpadding=5,
                 xoptions=gtk.EXPAND|gtk.FILL)
    i=i+1
    table.attach(justlabel("Not valid after:"), 
                 0, 1, i, i+1,yoptions=0,xpadding=5,
                 xoptions=gtk.EXPAND|gtk.FILL)
    table.attach(justlabel(dates[1]),
                  1, 2,i,i+1,yoptions=0,xpadding=5,
                 xoptions=gtk.EXPAND|gtk.FILL)

    dsecs=[]
    for j in range(len(dates)):
      dsecs.append(time.mktime(time.strptime(dates[j],'%b %d %H:%M:%S %Y GMT')))
    t=time.mktime(time.localtime())

    if dsecs[0]<t and dsecs[1]>t:
      tstring="(Currently valid)\n\nPress 'Next' to import this certificate"
    elif dsecs[0]>t:
       tstring="(Not valid yet)\nYou may still import this certificate by pressing 'Next'"
    elif dsecs[1]<t:
       tstring="(No longer valid)"
       next.set_sensitive(gtk.FALSE)
    
    i=i+1
    table.attach(justlabel(tstring),
                  1, 2,i,i+1,yoptions=0,xpadding=5,
                 xoptions=gtk.EXPAND|gtk.FILL)

    frame.show_all()

    #l.add_s("cn=%s,%s"%(dict['cn'][0],self.main_win.basedn),tuples)
    self.main_win.set_cwd(os.path.dirname(self.filename))

  def connect_status(self,params,page_box,next=None,prev=None,cancel=None):
    frame=gtk.GtkFrame()
    page_box.pack_start(frame)

    table=gtk.GtkTable(7,2,gtk.FALSE)
    table.set_border_width(3)
    table.set_col_spacings(5)
    table.set_row_spacings(5)
    frame.add(table)

    i=0
    table.attach(justlabel("Connecting to:"), 
                 0, 1, i, i+1,yoptions=0,xpadding=5,
                   xoptions=gtk.EXPAND|gtk.FILL)
    table.attach(justlabel("ldap://%s:%s"%(self.main_win.host,self.main_win.port)),
                 1, 2,i,i+1,yoptions=0,xpadding=5,
                   xoptions=gtk.EXPAND|gtk.FILL)
    table.show_all()    
    try:
      result=self.main_win.l.add("cn=%s,ou=People,%s"%(self.cert_dict['cn'][0],
                                    self.main_win.basedn),self.tuples)
      gtk_process_events()

      a=self.main_win.l.result(result)
      status="Certificate added"
    except ldap.LDAPError,v:
      status="Error: "+v[0]['desc']
      pass
    except Exception,v:
      status="Error: "+v[0]
      pass
    i=i+1
    table.attach(justlabel("Status:"), 
                 0, 1, i, i+1,yoptions=0,xpadding=5,
                   xoptions=gtk.EXPAND|gtk.FILL)
    table.attach(justlabel(status), 1, 2,i,i+1,yoptions=0,xpadding=5,
                   xoptions=gtk.EXPAND|gtk.FILL)
    gtk_process_events()


    frame.show_all()


  def pre_finish(self,params):
   self.fs.destroy()
   pass

class user_download:
  settings=None
  main_win=None
  def __init__(self,settings,main_win,result_id):
    self.settings=settings
    self.main_win=main_win
    self.win=gtk.GtkWindow(type=GTK.WINDOW_DIALOG)
    self.win.set_name("VO Groupman")
    self.win.set_title("VO Groupman: Downloading user info")
#    self.win.connect("destroy", self.quit)
    self.win.connect("delete_event", self.delete_cb)
    frame=gtk.GtkFrame()
    table=gtk.GtkTable(3,2,gtk.FALSE)
    table.set_border_width(3)
    table.set_col_spacings(5)
    table.set_row_spacings(5)
    self.result_id=result_id

    self.win.add(frame)
    frame.add(table)
    
    i=0

    i=i+1

    table.attach(justlabel("Connecting to:"), 
                 0, 1, i, i+1,yoptions=0,xpadding=5,
                   xoptions=gtk.EXPAND|gtk.FILL)
    table.attach(justlabel("ldap://%s:%s"%(self.main_win.host,self.main_win.port)),
                 1, 2,i,i+1,yoptions=0,xpadding=5,
                   xoptions=gtk.EXPAND|gtk.FILL)
    
    i=i+1
    self.label=justlabel("--")

    table.attach(justlabel("Status:"), 
                 0, 1, i, i+1,yoptions=0,xpadding=5,
                   xoptions=gtk.EXPAND|gtk.FILL)
    table.attach(self.label,
                 1, 2,i,i+1,yoptions=0,xpadding=5,
                   xoptions=gtk.EXPAND|gtk.FILL)
    self.win.show_all()
    while gtk.events_pending():
      gtk.mainiteration(block=gtk.FALSE)

  def delete_cb(self,widget,data):
    return
    
  def download(self):
    self.new_users={}
    num=0
    maxspin=5

    r=None
    oldr=None
    spin=0
    starttime=starttime=time.time()
    while num<25:
      num=num+1
      if r: oldr=r[0]
      else: oldr=None
      r=None
      gtk_process_events()
      try:
        r=self.main_win.l.result(self.result_id,timeout=1.0,all=0)

        if r and not r[0]:
          spin=spin+1
          if not oldr and spin>=maxspin: 
            spin=0
            self.label.set_text("Waiting for data...")
            while gtk.events_pending():
              gtk.mainiteration(block=gtk.FALSE)
            time.sleep(1)
#          continue
        else:
          num=0 # Reset num when results are received
      except ldap.TIMEOUT,msg:
        status="Error "+str(msg)
        raise
      except ldap.TIMELIMIT_EXCEEDED,msg:
        status="Error "+str(msg)
        rasie
      except Exception,msg:
        status="Error "+str(msg)
        raise
      if r:
        if r[1]: 
          status="User info downloaded"
          for j in xrange(len(r[1])):
            user=r[1][j]
            if user: self.new_users[user[0]]=user[1]
          while gtk.events_pending():
              gtk.mainiteration(block=gtk.FALSE)
          endtime=time.time()
          if endtime-starttime>1:
            self.label.set_text(user[0])
            starttime=time.time()
        else:
          continue
    return self.new_users

  def finish(self):
    self.win.destroy()
    while gtk.events_pending():
        gtk.mainiteration(block=gtk.FALSE)
  

class main_area:
  members={}
  users={}
  users_downloaded=0
  l=None
  userldap=None
  glist=None
  mlist=None
  settings=None
  host=None
  basedn=None
  bindname=None
  port=389
  host_cert=None
  basedn_cert=None
  bindname_cert=None
  port_cert=389

  def __init__(self):
    self.settings=Settings()
    self.settings.read()
    self.cwd=os.environ['PWD']+'/'
    self.create_main_win()

  def set_cwd(self,path):
    self.cwd=path
    if not self.cwd[-1]=='/':
      self.cwd=self.cwd+'/'

  def get_cwd(self):
    return self.cwd

  def get_selection(list="group"):
    if list=="group":
      return self.glist.selection
    elif list=="members":
      return self.mlist.selection
    else: return []

  def get_groups(self):
    return self.members

  def set_group(self,group_name,group_info):
    self.members[group]=group_info

  def get_users(self):
    if not self.users_downloaded:
      result=self.request_users(None,root='ou=People,%s'%self.basedn)
      status=gtk.FALSE
      dlg=user_download(self.settings,self,result)
      self.users=dlg.download()
      dlg.finish()
      del dlg
      self.users_downloaded=1
    return self.users

  def invert_list_selection(self,clist,col):
    if len(clist.selection)<clist.rows:
      clist.select_all()
    else:
      clist.unselect_all()
    
  def create_group_pane(self,table):
    swin=gtk.GtkScrolledWindow()
    swin.set_policy(GTK.POLICY_NEVER,GTK.POLICY_AUTOMATIC)
    i=0
    clist=gtk.GtkCList(cols=1,titles=["Groups"])
    clist.set_column_auto_resize(0,gtk.TRUE)
    clist.set_auto_sort(gtk.TRUE)
    clist.set_selection_mode(GTK.SELECTION_EXTENDED)
    clist.column_title_active(0)
    clist.connect("click-column",self.invert_list_selection)

    table.attach(swin, 
                 0, 1, i, i+1,xpadding=0,
                 xoptions=gtk.EXPAND|gtk.FILL,
                 yoptions=gtk.EXPAND|gtk.FILL)
    swin.add_with_viewport(clist)

    swin.show_all()
    return clist

  
  def create_members_pane(self,table):
    swin=gtk.GtkScrolledWindow()
    swin.set_policy(GTK.POLICY_NEVER,GTK.POLICY_AUTOMATIC)
    i=0
    clist=gtk.GtkCList(cols=1,titles=["Members"])
    clist.set_column_auto_resize(0,gtk.TRUE)
    clist.set_selection_mode(GTK.SELECTION_EXTENDED)
    clist.column_title_active(0)
    clist.connect("click-column",self.invert_list_selection)
    table.attach(swin, 
                 1, 2, i, i+1,xpadding=0,
                 xoptions=gtk.EXPAND|gtk.FILL,
                 yoptions=gtk.EXPAND|gtk.FILL)

    swin.add_with_viewport(clist)
    swin.show_all()
    return clist
  
  def ldap_init(self,host="localhost",port=389,
                basedn="dc=es, dc=net, c=us",bindname="cn=manager",
                passwd=None):
      l=ldap.initialize("ldap://%s:%d"%(host,port))
      l.simple_bind("%s,%s"%(bindname,basedn),passwd)
#      print "ldap_init: root='%s,%s', l=%s"%(bindname,basedn,repr(l))
#      print l.search_s("%s"%(basedn),ldap.SCOPE_SUBTREE,"(objectclass=*)")
      return l

  def ldap_init_group(self,host="localhost",port=389,
                    basedn="dc=es, dc=net, c=us",bindname="cn=manager",
                    passwd=None):
    
    self.l=self.ldap_init(host,port,basedn,bindname,passwd)
    self.host=host
    self.port=port
    self.basedn=basedn
    self.bindname=bindname
#    print "Init_group:l=",self.l
    return self.l
    
  def ldap_init_user(self,host="localhost",port=389,
                    basedn="dc=es, dc=net, c=us",bindname="cn=manager",
                    passwd=None):
    
    self.userldap=self.ldap_init(host,port,basedn,bindname,passwd)
    self.host_cert=host
    self.port_cert=port
    self.basedn_cert=basedn
    self.bindname_cert=bindname
    return self.userldap


  def request_groups(self,l,root=None):

    if not l: 
      my_l=self.l
    else: 
      my_l=l
#    print "request groups, l=",my_l
    if not root:
      root=self.basedn
#    print my_l.search_s(root,ldap.SCOPE_SUBTREE,"(objectclass=*)")
    return my_l.search(root,ldap.SCOPE_SUBTREE,"member=*")

  def request_users(self,l,root=None):
    if not l: 
      my_l=self.userldap
    else: 
      my_l=l
    if not root:
      root=self.basedn

#    print my_l.search_s(root,ldap.SCOPE_SUBTREE,"usercertificate;binary=*")
    return my_l.search(root,ldap.SCOPE_SUBTREE,"description=*")

  def fill_group_data(self,l,result_id,clist,state):
    num=0
#    print "result_id=",result_id
    if not state: 
      new_members={}
#      print "new_members=",new_members
    else: new_members=self.members={}
    if not l: return gtk.FALSE
    
    while num<5:
#      print "num=",num
      r=None
      try:
        r=l.result(result_id,timeout=2)
#        r=l.result(result_id)
#        print "r=",r
      except ldap.TIMEOUT,msg:
#        print "Timeout: ",msg
        return gtk.TRUE
      except ldap.NO_SUCH_OBJECT,msg:
        print "Error ",msg
        return gtk.FALSE
      except ldap.TIMELIMIT_EXCEEDED:
        self.members=new_members
        return gtk.FALSE
      for mem in r:
#        print mem
        new_members[mem[0]]=mem[1]
      num=num+1
    self.members=new_members
    if not r[1]:   # No more data entries
      return gtk.FALSE
    else:          # There is more to read, but the GUI may need time
      return gtk.TRUE

  
  def fill_group_list(self):
    self.glist.clear()
    for mem in self.members.keys():
      try:
        self.glist.append(self.members[mem]['dc'])
      except:
        self.glist.append(mem)
 
  def fill_members_list_cb(self):
    self.fill_members_list(self.glist,0,0,None,self.mlist)
    
  def fill_members_list(self,glist,row,col,event,mlist):
    sel=glist.selection
    if len(sel)==1:
      gname=glist.get_text(sel[0],0)
      gkeys=self.members.keys()
      for key in gkeys:
        if self.members[key]['dc'][0]==gname:
          members=self.members[key]['member']
          mlist.freeze()
          mlist.clear()
          for col in members:
             mlist.append([col])
          mlist.columns_autosize()
          mlist.thaw()
          break
    else:
      mlist.clear()   


  def fill_users(self,l,result_id,state):
    num=0
    if not state: 
      new_users={}
    else: new_users=self.users={}
    if not l: l=self.l
    while num<5:
      r=None
      try:
        r=l.result(result_id,timeout=4.0)
#        print "r=",r
      except ldap.TIMEOUT,msg:
        return gtk.TRUE
      except ldap.TIMELIMIT_EXCEEDED,msg:
        self.users=new_users
        return gtk.FALSE
      for user in r:
        new_users[user[0]]=user[1]
      num=num+1
    self.users=new_users
    if not r[1]:   # No more data entries
      return gtk.FALSE
    else:          # There is more to read, but the GUI may need time
      return gtk.TRUE


  def init_panes(self,members):
    self.members=members
    self.glist.clear()
    self.mlist.clear()
    status=gtk.TRUE
    self.fill_group_list()
    return status

  def quit(self,*args):
    self.settings.write()
    gtk.mainquit()
  
  def connect_cb(self,*args):
    cw=connect_wizard(self.settings,self)

  def new_group_cb(self,*args):
    ng=new_group(self.settings,self)

  def delete_group_cb(self,*args):
    dg=delete_group(self.settings,self)

  def edit_group_cb(self,*args):
    eg=group_edit(self.settings,self)

  def pem_import_cb(self,*args):
    pi=pem_import(self.settings,self)

  def update_users_cb(self,*args):
    update=connect_users_wizard(self.settings,self)


  def init_menu(self): 
       ag = gtk.GtkAccelGroup() 
       itemf = gtk.GtkItemFactory(gtk.GtkMenuBar, "<main>", ag) 
       self.win.add_accel_group(ag) 

#       itemf.create_items([
#                ('/_File',          None,	  None, 0, '<Branch>'),
#                ('/_File/_Open',    '<control>O', file_cb, 2, '')
       itemf.create_items([ 
               ('/_File',          None, 	 None, 0, '<Branch>'), 
               ('/_File/_Connect',     '<control>C', self.connect_cb, 1, ''), 
               ('/_File/_Import PEM',     '<control>I', self.pem_import_cb, 1, ''),
               ('/_File/_Update user info',    '<control>U', self.update_users_cb, 2, ''), 
##               ('/_File/_Save',    '<control>S', file_cb, 3, ''), 
##               ('/_File/Save _As', None,         file_cb, 4, ''), 
##               ('/_File/_Close',   None,         file_cb, 5, ''), 
#               ('/_File/sep1',     None,   file_cb, 0, '<Separator>'), 
               ('/_File/_Quit',    '<control>Q',    self.quit, 6, ''), 
               ('/_Group',          None,         None, 0, '<Branch>'), 
               ('/_Group/_New',   '<control>N', self.new_group_cb, 3, ''),
               ('/_Group/_Edit',     '<control>E', self.edit_group_cb, 1, ''), 
               ('/_Group/_Delete',    '<control>D', self.delete_group_cb, 2, ''), 

               ]) 
       self.w_menubar = itemf.get_widget('<main>') 
       self.main_vbox1.pack_start(self.w_menubar, expand=gtk.FALSE) 
       self.w_menubar.show()

  def create_main_win(self):
    self.win=gtk.GtkWindow()
    self.win.set_name("VO Groupman")
    self.win.set_title("Caltech VO group Manager")
    self.win.connect("destroy", self.quit)
    self.win.connect("delete_event", self.quit)
    self.win.set_usize(400, 300) 
    self.main_vbox1=gtk.GtkVBox(gtk.FALSE, 0)
    self.win.add(self.main_vbox1)
    self.main_vbox1.show()

    table=gtk.GtkTable(2,1,gtk.FALSE)
    table.set_border_width(3)
    table.set_col_spacings(5)
    table.set_row_spacings(5)
    self.init_menu()

    self.glist=self.create_group_pane(table,)
    self.mlist=self.create_members_pane(table)
    self.main_vbox1.pack_start(table,expand=gtk.TRUE,fill=gtk.TRUE)
    table.show()
    self.win.show()
    self.members={}
    self.glist.clear()
    self.mlist.clear()
    self.glist.connect("select-row",self.fill_members_list,self.mlist)
    self.glist.connect("unselect-row",self.fill_members_list,self.mlist)
    self.win.show()


m=main_area()
gtk.mainloop()
